#! /usr/local/bin/perl -s
#
# hl2html       --- convert hotlists to HTML
#
# Author: Oscar Nierstrasz 1/7/93
#
# This script and friends can be found at:
# http://iamwww.unibe.ch/~scg/Src/
#
#    -u suggested by Larry W. Virden (lvirden@cas.org)
#    NB: -u option prints the *first* entry for a given URL,
#    which may mean that an obsolete title is selected.
#
#    -l suggested by Andrew Tannebaum (trb@clam.com)
#    Warning: all users given as arguments are listed,
#    not just the ones for whom hot lists were found!
#    To fix this, would need to buffer the output before printing ...
#
#    Changes by Richard caley to add -g, -G, -st and -sl
#    Added reverse sort to -sh  by Alex Woo
#       Alex Woo, MS 227-6       http://ccf.arc.nasa.gov/~woo/woo.html
#       NASA Ames Research Center        woo@ames.arc.nasa.gov
#
#vn = "v1.0"; # 1/7/93
#vn = "v1.1"; # 1/8/93 -- Added -d and -u flags.
#vn = "v1.1"; # 4/8/93 -- Added -t flag (Larry Vierden).
#vn = "v1.2"; # 28/10/93 -- Added sort by title (-s)
#vn = "v1.3"; # 15/01/94 -- Added -l option (list users)
#vn = "hl2html v1.4"; # 21/3/94 -- changed $sig
#vn = "hl2html v1.5"; # 20/4/94 -- -g, -G, -st and -sl (by Richard Caley)
#vn = "hl2html v1.6"; # 15/8/94 -- reverse sort to -sh  by Alex Woo
$vn = "hl2html v1.7"; # 17/10/94 -- -htgrep option

$usg = 'Usage: hl2html [<options>] <user directory> ...
    -v -- verbose
    -u -- unique URLs only
    -t -- unique Title/URLs only
    -d -- list date of each entry
    -l -- list users in title
    -s -- sort by title
    -sl - sort by length of URL
    -sh - sort by host
    -g -- group things from same site
    -G -- like -g but dont show site name if there is only one entry
    -b -- no headers or footers (brief/bare output)
    -htgrep -- prepare output for htgrep
' ;


# This should match URLs to extract the hostname used for sorting.
# It eliminates some common first parts of names so that, say, ftp and
# www URLs at the same site will be put together.

$host_re='//([^@]*@)?(www\.|ftp\.|gopher\.|src\.|info\.|hoohoo\.)?([-a-zA-Z0-9.]*)(:[0-9]+)?/';

chop($date = `date +%d.%m.%y`);

$ftp = '<a href="http://iamwww.unibe.ch/~scg/Src/">';
$sig = "<i>This file was generated by $ftp$vn</a> on $date.</i>\n<p>\n";

($#ARGV >= 0) || die "$usg";

if ($l) { ($users = ": " . join(", ",@ARGV)) =~ s|\S*/||g; };
if ($htgrep) {
        $b = 1 ;
}

unless ($b) {
    print "<title>User Hotlists$users</title>\n";
    print "<h1>User Hotlists$users</h1>\n";
}

foreach $dir (@ARGV) {
    $hl = "$dir/.mosaic-hotlist-default";
    (-f $hl) || next;
    open(hl,$hl) || (warn "Can't open $hl\n", next);
    if ($v) { print STDERR "Trying $hl ..."; }
    ($user = $dir) =~ s|.*/||;
    $entry = "<h2>$user</h2>\n"
        unless $b;
    $entry .= "<ul>\n"
        unless $htgrep;
    $empty = 1;
    @hl = ();
    while(<hl>) {
        chop;
        /^ncsa-xmosaic-hotlist-format-1$/ && next;
        /^Default$/ && next;
        print $entry; $empty = 0; $entry = "";
        if (/^(\w+:[^ ]+) +(.*)/) {
            $url = $1;
            $when = $2;
            $title = <hl>; # read next line
            chop $title;
            if ($t && $seen{$url} && $seent{$title}) {
                 if ($v) { print STDERR "seen $title -- skipping it\n"; }
            }
            elsif ($u && $seen{$url}) {
                 if ($v) { print STDERR "seen $title/$url -- skipping it\n"; }
            }
            else {
                $item = "<li><a href=\"$url\">\n$title\n</a>\n";
                if ($d) { $item .= "<i>($when)</i>\n"; }
                if ($htgrep) { $item .= "\n"; }
                # if ($htgrep) { $item .= "<! $when !>\n\n"; }
                push(@hl,$item);
            }
            $seen{$url} = 1;
            $seent{$title} = 1;
        }
        # shouldn't happen:
        else { print STDERR " can't parse entry $_\n"; }
    }
    if ($g || $G) {&printBySite(@hl);}
    elsif ($sl||$sh||$s) { print sort byAll @hl; }
    else { print @hl; }
    if ($v) {
        if ($empty) { print STDERR " was empty -- ignored\n"; }
        else { print STDERR " got it!\n"; };
    }
    print "</ul>\n"
        unless $htgrep;
    close(hl);
}

print "\n$sig"
        unless $b;

sub byTitle {
        substr($a,index($a,"\n")) cmp substr($b,index($b,"\n")) ;
}

sub byHost {
        local ($as, $bs) = ($a, $b);
        local ($r);
        $as = $3
                if $a =~ m%$host_re% ;
        $bs = $3
                if $b =~ m%$host_re% ;
        # print STDERR "$a -> $as\n";
        #$as cmp $bs;
        join('.', reverse split(/\./, "L$as"))  cmp
                join('.',reverse split(/\./, "L$bs"));
}

sub byLen {
        rindex($a,"/",index($a,"\n")) cmp rindex($b,"/",index($b,"\n"));
}

sub byAll {
        local ($r);
        ($sh && ($r=&byHost()) != 0)
                || ($sl && ($r=&byLen()) != 0)
                || ($s && ($r=&byTitle()) != 0);
        $r;
}

sub printBySite
{
        local (@l) = @_;
        local ($site, $i, @entries);
        $site='';
        @entries=();
        foreach $i (sort byAll @l) {
                $i =~ m%$host_re% || ($3="local");
        #       print "--$1\n";
                if (!$site || $3 eq $site) {
                        push(@entries, $i);
                        $site=$3;
                } elsif ($G && $#entries == 0) {
                        print $entries[0];
                        @entries=($i);
                        $site=$3;
                } elsif ($#entries >= 0) {
                        print "<li> from $site <ul>\n";
                        print @entries;
                        print "</ul>\n";
                        @entries=($i);
                        $site=$3;
                }
        #       print "++$site\n";
        }
        if ($G && $#entries == 0) {
                print $entries[0];
                @entries=();
        }
        elsif ($#entries >= 0) {
                print "<li> from $site <ul>\n";
                print @entries;
                print "</ul>\n";
                @entries=();
        }
}


