#! /usr/local/bin/perl -s
#
# htls          --- generate index of HTML directory hierarchy
#
# Oscar Nierstrasz 9/9/94
# 20/10/94 -- added -htgrep option
#
# NB: It might be useful to have an option to automatically
# create toc.html files in each subdirectory ...
#
# This script and friends can be found at:
# http://iamwww.unibe.ch/~scg/Src/

$v = "htls v1.0";

if ($h) {
        print "Usage: htls <options> [<files|directories>]\n",
                "\t-ht\tonly list HTML files\n",
                "\t-d\tgenerate directory links too\n",
                "\t-r\treverse directory listing\n",
                "\t-nt\tsuppress title and heading\n",
                "\t-htgrep\tgenerate searchable output for htgrep\n";
        exit;
}

if ($htgrep) {
        $nt = 1; # htgrep => no titles or headings
}

if ($#ARGV >= $[) {
        &head(join(", ",@ARGV));
        foreach $arg (@ARGV) {
                &htls($arg);
        }
}
else {
        ($title = `pwd`) =~ s|.*/||;
        &head($title);
        &htls(".");
}

print "<hr>\n<i>This page was generated by $v.</i>\n\n"
        unless ($htgrep);

sub head {
        local($title) = @_;
        unless ($nt) {
                print "<title>Index of $title</title>\n";
                print "<h1>Index of $title</h1>\n";
        }
}

sub htls {
        local($dir) = @_;
        local($pre, $indent);
        local($path, $entry, @dirs, @list);

        if (-f $dir) {
                @list = ($dir);
                $pre = "";
        }
        elsif (-d $dir) {
                if (opendir(DIR,$dir)) {
                        @list = sort(readdir(DIR));
                        closedir(DIR);
                        if ($dir =~ /^\.$/) { $pre = ""; }
                        else { $pre = "$dir/"; }
                }
                else {
                        print "<i>unreadable</i>\n"
                                unless ($htgrep);
                }
        }
        else { return; } # ?!

        unless ($htgrep) {
                $indent = $pre;
                $indent =~ s|[^/]||g;
                $indent =~ s|/|  |g;
        }

        print "${indent}<ul>\n"
                unless ($htgrep);
        foreach $entry (@list) {
                if ($entry =~ /^\.+$/) {
                        next;
                }
                $path = "$pre$entry";
                if (-d $path) {
                        push(@dirs, $path);
                }
                elsif (-f $path) {
                        next if ($ht) && ($path !~ /\.html$/);
                        $title = &title($path);
                        print "${indent}<li><a href=$path>$title</a>\n";
                        print "\n" if ($htgrep);
                }
                # else ?!
        }
        if ($r) { @dirs = reverse(@dirs); }
        foreach $path (@dirs) {
                if ($d) {
                        print "${indent}<li><a href=$path><b>$path</b></a>\n";
                        print "\n" if ($htgrep);
                }
                else {
                        print "${indent}<li><b>$path</b>\n"
                                unless ($htgrep);
                }
                &htls("$path");
        }
        print "${indent}</ul>\n"
                unless ($htgrep);
}

# extract and return the title of a document
sub title {
        local($file) = @_;
        local($/) = "<";
        open(FILE,$file) || return "UNREADABLE";
        $title = $file; # default, in case no title found
        while (<FILE>) {
                /^title>\s*([^<]*)/i && ($title = $1) && last;
        }
        close(FILE);
        $title =~ s/\s*$//;
        return "$title";
}

__END__

